@@ -1,8 +1,12 @@
 from django.contrib import admin
+from django.contrib.gis.admin import OSMGeoAdmin
 
 from pxd_lingua.admin import TranslationsInlineAdmin
 from wcd_geo_db.modules.code_seeker.admin import CodesInliner
-from wcd_geo_db.modules.bank.db import Division, DivisionTranslation, DivisionCode
+from wcd_geo_db.modules.bank.db import (
+    Division, DivisionTranslation, DivisionCode,
+    DivisionPrefix, DivisionPrefixTranslation,
+)
 
 
 class DivisionTranslationsInlineAdmin(TranslationsInlineAdmin):
@@ -14,10 +18,10 @@ class DivisionCodeInlineAdmin(CodesInliner):
 
 
 @admin.register(Division)
-class DivisionAdmin(admin.ModelAdmin):
-    list_display = 'id', 'name', 'level', 'path', 'types'
-    list_filter = 'level',
-    search_fields = 'name', 'types', 'codes_set__code', 'codes_set__value'
+class DivisionAdmin(OSMGeoAdmin, admin.ModelAdmin):
+    list_display = 'id', 'name', 'label', 'level', 'path', 'types'
+    list_filter = 'level', 'prefix',
+    search_fields = 'name', 'label', 'types', 'codes_set__code', 'codes_set__value'
     autocomplete_fields = 'parent',
 
     inlines = (
@@ -38,3 +42,17 @@ class DivisionTranslationAdmin(admin.ModelAdmin):
     list_filter = 'language',
     search_fields = 'name', 'synonyms', 'language'
     autocomplete_fields = 'entity',
+
+
+class DivisionPrefixTranslationsInlineAdmin(TranslationsInlineAdmin):
+    model = DivisionPrefixTranslation
+
+
+@admin.register(DivisionPrefix)
+class DivisionPrefixAdmin(admin.ModelAdmin):
+    list_display = 'id', 'name', 'short',
+    search_fields = 'name', 'short',
+
+    inlines = (
+        DivisionPrefixTranslationsInlineAdmin,
+    )
