@@ -9,6 +9,7 @@ __all__ = 'Geometry', 'WithGeometryMixin',
 SRID = 4326
 
 
+# FIXME: This model should be removed.
 class Geometry(models.Model):
     class Meta:
         verbose_name = pgettext_lazy('wcd_geo_db:geometry', 'Geometry')
@@ -22,7 +23,7 @@ class Geometry(models.Model):
         srid=SRID, geography=True, spatial_index=True,
         null=True, blank=False
     )
-    polygon = geo_models.PolygonField(
+    polygon = geo_models.MultiPolygonField(
         verbose_name=pgettext_lazy('wcd_geo_db:geometry', 'Polygon'),
         srid=SRID, geography=True, spatial_index=True,
         null=True, blank=True
@@ -36,8 +37,20 @@ class WithGeometryMixin(models.Model):
     class Meta:
         abstract = True
 
+    # FIXME: This field should be removed.
     geometry = models.ForeignKey(
         Geometry,
         verbose_name=pgettext_lazy('wcd_geo_db:field', 'Geometry'),
         null=True, blank=True, on_delete=models.SET_NULL
     )
+
+    location = geo_models.PointField(
+        verbose_name=pgettext_lazy('wcd_geo_db:geometry', 'Location'),
+        srid=SRID, geography=True, spatial_index=True,
+        null=True, blank=False
+    )
+    polygon = geo_models.MultiPolygonField(
+        verbose_name=pgettext_lazy('wcd_geo_db:geometry', 'Polygon'),
+        srid=SRID, geography=True, spatial_index=True,
+        null=True, blank=True
+    )
