@@ -45,9 +45,7 @@ class NamedMixin(models.Model):
     class Meta:
         abstract = True
 
-    name = models.TextField(
-        verbose_name=pgettext_lazy('wcd_geo_db', 'Name')
-    )
+    name = models.TextField(verbose_name=pgettext_lazy('wcd_geo_db', 'Name'))
 
     def __str__(self):
         return self.name
