@@ -9,38 +9,78 @@ from wcd_geo_db.const import DivisionLevel
 from wcd_geo_db.modules.code_seeker.query import CodeSeekerQuerySet
 from pxd_tree.hierarchy import TreeQuerySet
 
-from ..dtos import DivisionDTO, GeometryDTO
+from ..dtos import DivisionDTO, ExtendedDivisionDTO, GeometryDTO, NamePrefixDTO
 from .base import QuerySet
 from .geometry import WithGeometryQuerySet
 from .search import SearchQueryParam, SearchQuerySet
+from .dto import DTOResolverQuerysetMixin
 
 
 __all__ = 'DivisionsQuerySet',
 
 QT = TypeVar('QT', bound='DivisionsQuerySet')
 VALUES_NAMES = (
-    'id', 'name', 'level', 'types', 'codes', 'parent_id', 'path',
-    'geometry__location', 'geometry__polygon',
+    'id', 'name', 'label', 'level', 'types', 'codes', 'parent_id', 'path',
 )
+EXTENDED_VALUES_NAMES = VALUES_NAMES + (
+    'prefix_id', 'location', 'polygon',
+)
+PREFIX_VALUES_NAMES = 'id', 'name', 'short',
 
 
-def as_dto(values: dict) -> DivisionDTO:
-    return DivisionDTO(
+# TODO: Rework.
+def extended_as_dto(values: dict, prefixes: dict) -> 'ExtendedDivisionDTO':
+    pget = (prefixes.get(values['prefix_id']) or {}).get
+
+    return ExtendedDivisionDTO(
         id=values['id'],
         name=values['name'],
+        label=values['label'],
+        prefix=NamePrefixDTO(name=pget('name', ''), short=pget('short', '')),
         level=DivisionLevel(values['level']),
         types=values['types'],
         codes=values['codes'],
         parent_id=values['parent_id'],
         path=list(values['path']),
         geometry=GeometryDTO(
-            location=values['geometry__location'],
-            polygon=values['geometry__polygon'],
+            location=values['location'],
+            polygon=values['polygon'],
         )
     )
 
 
+def extended_dto_resolver(query, **kwargs) -> List['ExtendedDivisionDTO']:
+    from wcd_geo_db.modules.bank.db import DivisionPrefix
+
+    values = query.values(*EXTENDED_VALUES_NAMES)
+    prefixes = {
+        x['id']: x
+        for x in DivisionPrefix.objects.all().values(*PREFIX_VALUES_NAMES)
+    }
+
+    return [extended_as_dto(values, prefixes=prefixes) for values in values]
+
+
+def base_as_dto(values: dict) -> 'DivisionDTO':
+    return DivisionDTO(
+        id=values['id'],
+        name=values['name'],
+        label=values['label'],
+        level=DivisionLevel(values['level']),
+        types=values['types'],
+        codes=values['codes'],
+        parent_id=values['parent_id'],
+        path=list(values['path']),
+    )
+
+
+def base_dto_resolver(query, **kwargs) -> List['DivisionDTO']:
+    values = query.values(*VALUES_NAMES)
+    return [base_as_dto(values) for values in values]
+
+
 class DivisionsQuerySet(
+    DTOResolverQuerysetMixin,
     WithGeometryQuerySet,
     CodeSeekerQuerySet,
     TreeQuerySet,
@@ -51,6 +91,12 @@ class DivisionsQuerySet(
     SEARCH_QUERY_MIN_RANK: float = 0.1
     SEARCH_QUERY_RANK_WEIGHTS: Sequence[float] = [0.2, 0.4, 0.8, 1]
 
+    # TODO: Rework.
+    dto_default_resolvers = {
+        DivisionDTO: base_dto_resolver,
+        ExtendedDivisionDTO: extended_dto_resolver,
+    }
+
     def _search_rank_order(self: QT, query: SearchQueryParam) -> QT:
         q = query.get('query')
         min_rank = query.get('min_rank', self.SEARCH_QUERY_MIN_RANK)
@@ -137,13 +183,10 @@ class DivisionsQuerySet(
 
         return self._search_rank_order(query)
 
-
-    def as_dtos(self) -> List[DivisionDTO]:
-        return [as_dto(values) for values in self.values(*VALUES_NAMES)]
-
     def general_filter(
         self: QT,
         parent_ids: Optional[Sequence[int]] = None,
+        labels: Optional[Sequence[str]] = None,
         levels: Optional[Sequence[DivisionLevel]] = None,
         types: Optional[Sequence[str]] = None,
         **kw
@@ -151,6 +194,7 @@ class DivisionsQuerySet(
         q = (
             super().general_filter(**kw)
             .optional_in('parent_id', parent_ids)
+            .optional_in('label', labels)
             .optional_in('level', levels)
             .optional_overlap('types', types)
         )
