@@ -19,6 +19,8 @@ def in_areas_Q(
 ):
     q = Q()
 
+    # FIXME: Inefficient query.
+    # ReSearch: https://postindustria.com/postgresql-geo-queries-made-easy/
     for point, distance in areas:
         q |= Q(**{f'{field_name}__distance__lte': (point, distance)})
 
@@ -28,7 +30,7 @@ def in_areas_Q(
 class WithGeometryQuerySet(QuerySet):
     def in_location_areas(self, areas: Sequence[Area]):
         return self.filter(in_areas_Q(
-            areas, field_name='geometry__location'
+            areas, field_name='location'
         ))
 
     def general_filter(
