@@ -1,12 +1,17 @@
 from typing import Any, Dict, Optional, Sequence
+from django.contrib.gis.geos import Polygon, Point, GEOSGeometry, MultiPolygon
 from itertools import chain
 from pxd_postgres.ltree import LtreeValue
-from wcd_geo_db.modules.bank.db import Division, DivisionCode, DivisionTranslation
+from wcd_geo_db.modules.bank.db import Division, DivisionCode, DivisionTranslation, DivisionPrefix
+from wcd_geo_db.modules.bank.db.geometry import SRID
 from wcd_geo_db.modules.code_seeker import CodeSeekerRegistry
 
 from .code_seeker import get_code_seeker_registry
 from .code_mapper import CodeMapper
-from .dtos import DivisionItem, DivisionTranslationItem
+from .dtos import (
+    DivisionItem, DivisionTranslationItem,
+    Point as PointDTO, Polygon as PolygonDTO,
+)
 from .merger import MergeCommiter, inject_synonyms
 
 
@@ -57,20 +62,63 @@ def update_existing_codes(codes, existing_codes):
     return existing_codes
 
 
+def make_point(point_data: Optional[PointDTO]) -> Optional[Point]:
+    if not point_data:
+        return None
+
+    latitude, longitude = point_data
+
+    return Point(x=longitude, y=latitude, srid=SRID)
+
+
+def make_polygon(polygon_data: Optional[PolygonDTO]) -> Optional[GEOSGeometry]:
+    if not polygon_data:
+        return None
+
+    geom = GEOSGeometry(polygon_data)
+
+    if isinstance(geom, Polygon):
+        geom = MultiPolygon(geom)
+
+    return geom
+
+
+class NamePrefixResolver:
+    def __init__(self):
+        self.cache: Dict[str, DivisionPrefix] = {}
+
+    def get(self, name_prefix: str) -> DivisionPrefix:
+        if not name_prefix:
+            return None
+
+        if name_prefix in self.cache:
+            return self.cache[name_prefix]
+
+        self.cache[name_prefix], _ = DivisionPrefix.objects.get_or_create(
+            name=name_prefix
+        )
+
+        return self.cache[name_prefix]
+
+
 def merge_divisions(
     items: Sequence[DivisionItem],
     change_path: bool = True,
     change_level: bool = True,
     change_types: bool = True,
+    change_geo: bool = True,
     should_create: bool = True,
+    override_name: bool = False,
+    override_name_prefix: bool = False,
     lookup_on_singular_name_equality=False,
     strict_types_change: bool = False,
 ):
     d = MergeCommiter(Division, update_fields=(
-        ('name', 'types', 'codes', 'level', 'synonyms',)
+        ('name', 'types', 'codes', 'level', 'synonyms', 'location', 'polygon', 'prefix')
         +
         (('parent_id', 'path') if change_path else ())
     ))
+    name_prefix = NamePrefixResolver()
     # dc = MergeCommiter(DivisionCode, update_fields=('code', 'value'))
 
     divisions = find_by_codes(get_code_seeker_registry(), items)
@@ -89,16 +137,22 @@ def merge_divisions(
             continue
 
         path = [(x.id if x else x) for x in path]
+        prefix = name_prefix.get(item.get('name_prefix'))
         parent_id = path[-1] if len(path) > 0 else None
+        location = make_point(item.get('location'))
+        polygon = make_polygon(item.get('polygon'))
 
         if eqs is None:
             if should_create:
                 d.create(Division(
-                    name = item['name'],
-                    codes = update_existing_codes(item_codes, {}),
-                    types = item['types'],
-                    level = item['level'],
-                    path = LtreeValue(path),
+                    name=item['name'],
+                    codes=update_existing_codes(item_codes, {}),
+                    types=item['types'],
+                    level=item['level'],
+                    prefix=prefix,
+                    location=location,
+                    polygon=polygon,
+                    path=LtreeValue(path),
                     parent_id=parent_id,
                 ))
             else:
@@ -108,6 +162,15 @@ def merge_divisions(
             eqs.codes = update_existing_codes(item_codes, eqs.codes or {})
             eqs.parent_id = parent_id
 
+            if location and change_geo:
+                eqs.location = location
+
+            if (override_name_prefix or eqs.prefix_id is None) and prefix is not None:
+                eqs.prefix = prefix
+
+            if polygon and change_geo:
+                eqs.polygon = polygon
+
             if change_level:
                 eqs.level = item['level']
 
@@ -117,7 +180,13 @@ def merge_divisions(
                 else:
                     eqs.types = list(set((eqs.types or []) + item['types']))
 
-            inject_synonyms(eqs, item['name'])
+            synonym_name = item['name']
+
+            if override_name:
+                synonym_name = eqs.name
+                eqs.name = item['name']
+
+            inject_synonyms(eqs, synonym_name, item.get('synonyms') or '')
 
             d.update(eqs)
 
@@ -130,7 +199,8 @@ def merge_divisions(
 
 def merge_division_translations(
     language: str,
-    items: Sequence[DivisionTranslationItem]
+    items: Sequence[DivisionTranslationItem],
+    override_name: bool = False,
 ):
     creations = []
     updates = []
@@ -163,12 +233,20 @@ def merge_division_translations(
                 entity_id=entity.id
             ))
         else:
-            inject_synonyms(existing, item['name'], item.get('synonyms') or '')
+            synonym_name = item['name']
+
+            if override_name:
+                synonym_name = existing.name
+                existing.name = item['name']
+
+            inject_synonyms(existing, synonym_name, item.get('synonyms') or '')
             updates.append(existing)
 
     print(merge_failures)
 
     DivisionTranslation.objects.bulk_create(creations, ignore_conflicts=True)
     DivisionTranslation.objects.bulk_update(
-        updates, fields=('synonyms',)
+        updates, fields=(
+            ('name', 'synonyms',) if override_name else ('synonyms',)
+        )
     )
