@@ -12,17 +12,20 @@ class CodesItem(TypedDict):
     codes: List[Tuple[str, Any]]
 
 
-class Point(TypedDict):
-    latitude: Decimal
-    longitude: Decimal
+Point = Tuple[Decimal, Decimal]
+"""[latitude, longitude]"""
+Polygon = str
 
 
 class DivisionItem(CodesItem, TypedDict):
     path: List[Tuple[str, Any]]
     name: Optional[str]
+    name_prefix: Optional[str]
+    synonyms: Optional[str]
     types: List[DivisionType]
     level: DivisionLevel
-    point: Optional[Point]
+    location: Optional[Point]
+    polygon: Optional[Polygon]
 
 
 class DivisionTranslationItem(TypedDict):
