@@ -8,7 +8,7 @@ __all__ = 'BaseImportStage',
 
 
 class BaseImportStage(models.TextChoices):
-    INITIAL = ProcessStage.INITIAL
+    INITIAL = ProcessStage.INITIAL.value, ProcessStage.INITIAL.label
     UPLOADING = '002000-uploading', pgettext_lazy('wcd_geo_db_sources:source', 'Uploading')
     PARSING = '003000-parsing', pgettext_lazy('wcd_geo_db_sources:source', 'Parsing')
     MERGE = '004000-merge', pgettext_lazy('wcd_geo_db_sources:source', 'Merge')
